--
-- VHDL Architecture Cosmix_lib.SPI0.arch_name
--
-- Created:
--          by - goodrick.UNKNOWN (PCCO)
--          at - 11:05:55 22/08/2005
--
-- using Mentor Graphics HDL Designer(TM) 2003.2 (Build 28)
--
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;

ENTITY SPI_DAC IS
  PORT( 
    GCK   : IN     std_logic;
    SPCK  : IN     std_logic;
    Rst   : IN     std_logic;
    SPDI  : IN     std_logic;
    SPCSn : IN     std_logic;
    Outz  : OUT    std_logic_vector (15 DOWNTO 0);
    SDCSn : OUT    std_logic;
    SDCK  : OUT    std_logic;
    SDCLn : OUT    std_logic;
    SDDI  : OUT    std_logic;
    SVar  : OUT    std_logic_vector (3 DOWNTO 0)
  );

-- Declarations

END SPI_DAC ;

--
ARCHITECTURE Mk3 OF SPI_DAC IS
 --** Synchronous version **
   type StateType is (StandBy,NotMe,GetByte1,GetByte2);
   signal PresentState, NextState : StateType;
   signal SerialReg, NextSerialReg: std_logic_vector (15 downto 0) ;
 --** signal SGCk: std_logic; -- re-sync'd SPCK
   signal BitCount, NextBitCount: integer ;
 --** Counter for use in State M/C
   signal SPCKL,SPCKLL : std_logic ;
  
BEGIN
   
   SDCLn <= '1'   ;
   SDDI  <= SPDI  ;
   Outz  <= SerialReg ;
--    
   StateMC: process (BitCount,PresentState,SPCSn,SPDI)
   begin
      case PresentState is
--
         when StandBy =>
            SVar <= "0001"; -- DeBug
            SDCSn <= '1'  ;
            SDCK  <= '0'  ;
            if(SPCSn = '1') then -- restart
               NextState     <= StandBy   ;
               NextBitCount  <= 0         ;              
            elsif((BitCount >= 8)and(SPCK = '0')) then
               NextBitCount  <= 0         ;
               NextSerialReg <= SerialReg ;
               if(SerialReg(7 DOWNTO 0) = "10100000") then
                  NextState  <= GetByte1  ;
               else
                  NextState  <= NotMe     ;
               end if;
            elsif ((SPCKL='1') and (SPCKLL='0')) then
               NextState     <= PresentState                  ;            
               NextBitCount  <= BitCount + 1                  ;            
               NextSerialReg <= SerialReg(14 downto 0) & SPDI ;
            else
               NextState     <= PresentState   ;
               NextBitCount  <= BitCount       ;
               NextSerialReg <= SerialReg      ;
            end if;
--
         when NotMe =>
            SVar <= "0010"; -- DeBug
            SDCSn <= '1'  ;
            SDCK  <= '0'  ;
            NextBitCount <= 0                  ;
            if(SPCSn = '1') then -- restart
               NextState     <= StandBy        ;
               NextSerialReg <= (others => '0');
            else
               NextState     <= PresentState   ;
               NextSerialReg <= SerialReg      ;
            end if ;
--
         when GetByte1 =>
            SVar <= "0100"; -- DeBug
            SDCSn <= SPCSn  ;
            SDCK  <= SPCK ;
            if(SPCSn = '1') then -- restart
               NextState     <= StandBy   ;
               NextBitCount  <= 0         ;
            elsif ((BitCount >= 8)and(SPCK = '0')) then
               NextState     <= GetByte2  ;
               NextBitCount  <= 0         ;
               NextSerialReg <= SerialReg ;
            elsif ((SPCKL='1') and (SPCKLL='0')) then
               NextState     <= PresentState                  ;            
               NextBitCount  <= BitCount + 1                  ;
               NextSerialReg <= SerialReg(14 downto 0) & SPDI ;
            else
               NextState     <= PresentState   ;
               NextBitCount  <= BitCount       ;
               NextSerialReg <= SerialReg      ;
            end if;
--
         when GetByte2 =>
            SVar <= "1000"; -- DeBug
            SDCSn <= SPCSn  ;
            SDCK  <= SPCK ;
            if(SPCSn = '1') then -- restart
               NextState     <= StandBy   ;
               NextBitCount  <= 0         ;
            elsif ((BitCount >= 8)and(SPCK = '0')) then
               NextState     <= StandBy   ;
               NextBitCount  <= 0         ;
               NextSerialReg <= SerialReg ;
            elsif ((SPCKL='1') and (SPCKLL='0')) then
               NextState     <= PresentState                  ;            
               NextBitCount  <= BitCount + 1                  ;
               NextSerialReg <= SerialReg(14 downto 0) & SPDI ;
            else
               NextState     <= PresentState   ;
               NextBitCount  <= BitCount       ;
               NextSerialReg <= SerialReg      ;
            end if;
--
     end case ;    
   end process StateMC ;
--    
   StateClock: process (GCK,Rst,SPCK)
   begin
      if(Rst ='1') then
         SPCKLL       <= SPCK            ;
         SPCKL        <= SPCK            ;
         PresentState <= StandBy         ;
         SerialReg    <= (others => '0') ;
         BitCount     <= 0 ;
      elsif( rising_edge(GCK)) then
         SPCKLL       <= SPCKL           ;
         SPCKL        <= SPCK            ;
         PresentState <= NextState       ;
         SerialReg    <= NextSerialReg   ;
         BitCount     <= NextBitCount    ;
      end if ;
   end process StateClock ;
--    
   
END Mk3;

