--
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;

ENTITY hit_stretch IS
   PORT( 
     clk       : IN     std_logic;
     reset     : in     std_logic;
     pulse     : IN     std_logic;
     out_width : in     std_logic_vector(3 downto 0);
     hit       : OUT    std_logic
   );

-- Declarations

END hit_stretch ;

--
ARCHITECTURE v0 OF hit_stretch IS

  signal hit_loc : std_logic;
  signal hit_timeout : unsigned(3 downto 0);
  signal hit_reset : std_logic;

  type stretch_state_t is (ss_idle,ss_wait_timeout);
  signal stretch_state: stretch_state_t;
  
  BEGIN

    hit <= hit_loc;
    
-- Latch the incoming pulse, will be reset on expiry of timeout

    process (pulse,hit_reset)
     begin
        if(hit_reset = '1') then
           hit_loc <= '0';
        elsif(pulse'event and pulse='1') then
           hit_loc  <= '1';
        end if;
     end process;

-- When the latched hit is asserted, generate timeout reset
     
    process (clk,reset)
     begin
        if(reset = '1') then
          
           hit_reset <= '1';
           hit_timeout <= (others=>'0');
           stretch_state <= ss_idle;
           
        elsif(clk'event and clk='1') then
          
           hit_reset <= '0';

           case stretch_state is
             
             when ss_idle  =>
             if (hit_loc='1') then
               hit_timeout <= unsigned(out_width);
               stretch_state <= ss_wait_timeout;
             end if;

             when ss_wait_timeout =>
               if (hit_timeout = 0) then
                 stretch_state <= ss_idle;
                 hit_reset <= '1';
               else
                 hit_timeout <= hit_timeout - 1;
               end if;
               
             when others =>
               hit_reset <= '1';
               stretch_state <= ss_idle;
                            
           end case;
           
        end if;
     end process;

END ARCHITECTURE v0;

